/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#ifndef TCP_SEGMENT_FACTORY_H
#define TCP_SEGMENT_FACTORY_H

#include <stdint.h>

#include <iostream>

#include <boost/asio.hpp>
#include <boost/asio/ip/tcp_raw_protocol.hpp>

#include "tcp/tcpsegment.h"

//-----------------------------------------------------------------------------
// TcpSegmentFactory
//-----------------------------------------------------------------------------

/**
 * @brief Class which constructs TCP Segments, hiding from the class users the
 * underlying details on how to build each segment.
 */
class TcpSegmentFactory
{
public:
    static TcpSegmentItem create_tcp_segment(
            const boost::asio::ip::tcp_raw_protocol::socket::protocol_type &protocol,
            std::istream &is
    );
    static TcpSegmentItem create_tcp_segment_syn_request(
            const boost::asio::ip::tcp_raw_protocol::socket::protocol_type &protocol,
            const boost::asio::ip::address &source_address,
            const boost::asio::ip::address &destination_address,
            const uint16_t source_port,
            const uint16_t destination_port,
            const uint16_t sequence_number
    );

private:
    static TcpSegmentItem create_tcp_ipv4_segment_syn_request(
            const boost::asio::ip::address &source_address,
            const boost::asio::ip::address &destination_address,
            const uint16_t source_port,
            const uint16_t destination_port,
            const uint16_t sequence_number
    );
    static TcpSegmentItem create_tcp_ipv6_segment_syn_request(
            const boost::asio::ip::address &source_address,
            const boost::asio::ip::address &destination_address,
            const uint16_t source_port,
            const uint16_t destination_port,
            const uint16_t sequence_number
    );

};

#endif // TCP_SEGMENT_FACTORY_H
